<?php
/**
 * Plugin Name: Disable All Automatic Updates and Theme Editors
 * Description: Disable All WordPress Updates, File Editors and Gutenberg is a handy tool that allows you to disable all types of automatic updates on WordPress, plugins and themes, theme-plugin editors and Gutenberg editor. Besides, it also helps you with enabling 'debugging' mode, debug log and display errors when you're troubleshooting your website.
 * Plugin URI:  URL to info about plugin.
 * Author URI:  https://www.saffiretech.com
 * Author:      SaffireTech
 * Text Domain: disable-wordpress-automatic-updates-theme-editors
 * Domain Path: /languages
 * Requires at least: 5.0
 * Tested up to: 6.0.1
 * Requires PHP: 7.2.0
 * License:     GPLv3
 * License URI: URI: https://www.gnu.org/licenses/gpl-3.0.html
 * Version:     1.0.1
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Load required plugin file.
 */
function dawufeg_load_required_files() {
	require_once dirname( __FILE__ ) . '/library/wp-config-transformer/src/WPConfigTransformer.php';
	require_once dirname( __FILE__ ) . '/include/dawufeg-gutenburg.php';
	require_once dirname( __FILE__ ) . '/include/dawufeg-settings-function.php';
	require_once dirname( __FILE__ ) . '/include/dawufeg-theme-plugin-updates.php';
	require_once dirname( __FILE__ ) . '/include/dawufeg-updates.php';
	require_once dirname( __FILE__ ) . '/include/dawufeg-debug.php';
}
add_action( 'init', 'dawufeg_load_required_files' );

/**
 * Loads script files and css file.
 */
function dawufeg_load_admin_file() {
	global $pagenow;
	// phpcs:disable WordPress.Security.NonceVerification.Recommended
	if ( isset( $_GET['page'] ) ) {
		if ( 'options-general.php' === $pagenow && 'dawufeg-disable-menu' === sanitize_text_field( wp_unslash( $_GET['page'] ) ) ) {
			wp_enqueue_script( 'jquery' );
			wp_enqueue_style( 'dawufeg-custom-toggle', plugins_url( 'assets/css/custom-setting.css', __FILE__ ), array(), '1.0.0' );
			wp_register_script( 'dawufeg_setting_js', plugins_url( 'assets/js/custom-setting.js', __FILE__ ), array( 'jquery', 'wp-i18n' ), '1.0.0', false );
			wp_enqueue_script( 'dawufeg_setting_js' );
			wp_set_script_translations( 'dawufeg_setting_js', 'disable-wordpress-automatic-updates-theme-editors', plugin_dir_path( __FILE__ ) . 'languages/' );
			wp_localize_script(
				'dawufeg_setting_js',
				'dawufeg_data',
				array(
					'url'         => admin_url( 'admin-ajax.php' ),
					'dawufeg_per' => dawufeg_config_file_permission_check(),
				)
			);
		}
	}
	load_plugin_textdomain( 'disable-wordpress-automatic-updates-theme-editors', false, basename( dirname( __FILE__ ) ) . '/languages/' );
}
add_action( 'admin_init', 'dawufeg_load_admin_file' );

/**
 * For adding shortcut setting link to plugin page.
 *
 * @param array $links links Plugin links on plugins.php.
 * @return array
 */
function dawufeg_plugin_action_links_callback( $links ) {
	$settinglinks = array(
		'<a href="' . admin_url( 'options-general.php?page=dawufeg-disable-menu' ) . '">' . __( 'Settings', 'disable-wordpress-automatic-updates-theme-editors' ) . '</a>',
	);
	return array_merge( $settinglinks, $links );
}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'dawufeg_plugin_action_links_callback', 10, 1 );

/**
 * Gets config file location.
 *
 * @return string config file path.
 */
function dawufeg_get_config_path() {
	$config_path = ABSPATH . 'wp-config.php';
	if ( ! file_exists( $config_path ) ) {
		if ( @file_exists( dirname( ABSPATH ) . '/wp-config.php' ) && ! @file_exists( dirname( ABSPATH ) . '/wp-settings.php' ) ) {
			$config_path = dirname( ABSPATH ) . '/wp-config.php';
		}
	}
	return apply_filters( 'dawufeg_wp_debugging_config_path', $config_path );
}
add_filter( 'dawufeg_wp_debugging_config_path', 'dawufeg_plugin_path_callback', 10, 1 );

/**
 * Returns config file path.
 *
 * @param string $config_path .
 * @return string file path
 */
function dawufeg_plugin_path_callback( $config_path ) {
	return esc_html( $config_path );
}

/**
 * Show notice on checking write permission of wp-config file.
 */
function dawufeg_admin_notices_callback() {
	global $pagenow;
	$config_file_path = strval( dawufeg_get_config_path() ); // get config file path.

	// display notice on plufgin setting page.
	if ( isset( $_GET['page'] ) ) {
		if ( 'options-general.php' === $pagenow && 'dawufeg-disable-menu' === sanitize_text_field( wp_unslash( $_GET['page'] ) ) ) {
			if ( ! is_writable( $config_file_path ) ) { ?>
				<div class="notice notice-error is-dismissible"> 
					<p><strong><?php esc_html_e( 'You do not have write permission to wp_config file some options will not work !', 'disable-wordpress-automatic-updates-theme-editors' ); ?></strong></p>
				</div>
				<?php
			}
		}
	}
}
add_action( 'admin_notices', 'dawufeg_admin_notices_callback' );

/**
 * Error notice if content is not written.
 */
function dawufeg_error_notices_callback() {
	?>
	<div class="notice notice-error is-dismissible"> 
		<p><strong><?php esc_html_e( 'unable to put content in config file', 'disable-wordpress-automatic-updates-theme-editors' ); ?></strong></p>
	</div>
	<?php
}

/**
 * Error notice if content is not written.
 */
function dawufeg_error_update_notices_callback() {
	?>
	<div class="notice notice-error is-dismissible"> 
		<p><strong><?php esc_html_e( 'unable to update content', 'disable-wordpress-automatic-updates-theme-editors' ); ?></strong></p>
	</div>
	<?php
}

// settings function callbacks.
add_action( 'init', 'dawufeg_update_setting_callback', 11 );
add_action( 'init', 'dawufeg_theme_plugin_setting_callback', 11 );
add_action( 'init', 'dawufeg_guternburg_setting_callback', 11 );
add_action( 'init', 'dawufeg_debug_setting_callback', 11 );
