<?php
/**
 * Dynamically change capability for theme and plugin.
 *
 * @param string $allcaps .
 * @param string $caps .
 * @param mixed  $args .
 * @param string $user .
 * @return array
 */
function dawufeg_change_theme_plugin_cap( $allcaps, $caps, $args, $user ) {

	// list of capability for file upload.
	$user_cap = array( 'install_themes', 'delete_themes', 'install_plugins', 'delete_plugins' );

	// disable file uploads and theme upload button.
	if ( get_option( 'dawufeg_uploads' ) || 1 === get_option( 'dawufeg_uploads' ) ) {
		foreach ( $user_cap as $cap ) {
			$allcaps[ $cap ] = false;
		}
	}

	// disable file and theme editor.
	if ( get_option( 'dawufeg_editor' ) || 1 === get_option( 'dawufeg_editor' ) ) {
		$allcaps['edit_plugins'] = false;
		$allcaps['edit_themes']  = false;
	} else {
		$allcaps['edit_plugins'] = true;
		$allcaps['edit_themes']  = true;
	}
	return $allcaps;
}

/**
 * Change constant on radio button enabled/disabled for file edit.
 */
function dawufeg_theme_plugin_edit_constant_change() {

	// on first time plugin install enbale option for file edit.
	if ( 'first' !== get_option( 'edit_filter_first_time' ) ) {
		update_option( 'dawufeg_editor', 1 );
		add_option( 'edit_filter_first_time', 'first' );
	}
	add_filter( 'user_has_cap', 'dawufeg_change_theme_plugin_cap', 10, 4 );
}

/**
 * Change user capability.
 * disallow plugin manual updates.
 */
function dawufeg_theme_plugin_modification_constant_change() {

	// on first time plugin install enbale option for file mod.
	if ( 'first' !== get_option( 'upload_filter_first_time' ) ) {
		update_option( 'dawufeg_uploads', 1 );
		add_filter( 'site_transient_update_plugins', '__return_true' );
		add_filter( 'site_transient_update_themes', '__return_true' );
		add_option( 'upload_filter_first_time', 'first' );
	} else {
		if ( get_option( 'dawufeg_uploads' ) || 1 === get_option( 'dawufeg_uploads' ) ) {
			add_filter( 'site_transient_update_plugins', '__return_true' );
			add_filter( 'site_transient_update_themes', '__return_true' );
		}
	}
	add_filter( 'user_has_cap', 'dawufeg_change_theme_plugin_cap', 20, 4 );
}

/**
 *  Handels theme & plugin updates settings.
 */
function dawufeg_theme_plugin_setting_callback() {
	// phpcs:disable WordPress.Security.NonceVerification.Recommended

	$config_file_path = strval( dawufeg_get_config_path() ); // config file path.

	// if config file exist and have write permission.
	if ( $config_file_path && file_exists( $config_file_path ) ) {
		if ( is_writable( $config_file_path ) && current_user_can( 'manage_options' ) ) {
			$config_transformer = new \WPConfigTransformer( $config_file_path );
		}
	}

	// if constant is there in wp-config file and defined then works with wp-config file and update that constant value by replacing that defination from wp-config file.
	if ( is_writable( $config_file_path ) && current_user_can( 'manage_options' ) ) {

		if ( defined( 'DISALLOW_FILE_EDIT' ) ) {

			// if constant is set to true update chekbox and on false change make constant true.
			if ( 'first' !== get_option( 'edit_first_time' ) ) {

				update_option( 'dawufeg_editor', 1 ); // default enable editor option.

				// if invalid value found in constant replacing that to default true in wp-config file.
				if ( ! in_array( strtolower( strval( constant( 'DISALLOW_FILE_EDIT' ) ) ), array( '1', '' ), true ) && defined( 'DISALLOW_FILE_EDIT' ) ) {

					// updating constant to true.
					try {
						$config_transformer->update( 'constant', 'DISALLOW_FILE_EDIT', 'true', array( 'raw' => true ) );
						update_option( 'dawufeg_editor', 1 );
					} catch ( \Exception $ex ) {
						add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
						update_option( 'dawufeg_editor', 0 );
					}
				} elseif ( in_array( strtolower( strval( constant( 'DISALLOW_FILE_EDIT' ) ) ), array( '1', '' ), true ) && defined( 'DISALLOW_FILE_EDIT' ) ) { // on valid value of constant.

					// if file edit is set false defaulting to true otherwise just update switch value.
					if ( 1 === intval( constant( 'DISALLOW_FILE_EDIT' ) ) ) {
						update_option( 'dawufeg_editor', 1 );
					} else {

						// updating constant to true.
						try {
							$config_transformer->update( 'constant', 'DISALLOW_FILE_EDIT', 'true', array( 'raw' => true ) );
							update_option( 'dawufeg_editor', 1 );
						} catch ( \Exception $ex ) {
							add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
							update_option( 'dawufeg_editor', 0 );
						}
					}
				} else {  // if constant is not defined work with function that change constant value.
					update_option( 'dawufeg_editor', 1 );
					dawufeg_theme_plugin_edit_constant_change();
				}
				add_option( 'edit_first_time', 'first' );
			} elseif ( 'first' === get_option( 'edit_first_time' ) && ! isset( $_GET['settings-updated'] ) ) {

				// if invalid value found in constant replacing that to default true in wp-config file.
				if ( ! in_array( strtolower( strval( constant( 'DISALLOW_FILE_EDIT' ) ) ), array( '1', '' ), true ) && defined( 'DISALLOW_FILE_EDIT' ) ) {

					// update constant value to true.
					try {
						$config_transformer->update( 'constant', 'DISALLOW_FILE_EDIT', 'true', array( 'raw' => true ) );
						update_option( 'dawufeg_editor', 1 );
					} catch ( \Exception $ex ) {
						add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
						update_option( 'dawufeg_editor', 0 );
					}
				} elseif ( in_array( strtolower( strval( constant( 'DISALLOW_FILE_EDIT' ) ) ), array( '1', '' ), true ) && defined( 'DISALLOW_FILE_EDIT' ) ) {

					// if file edit is set false or true update option.
					if ( 1 === intval( constant( 'DISALLOW_FILE_EDIT' ) ) ) {
						update_option( 'dawufeg_editor', 1 );
					} else {
						update_option( 'dawufeg_editor', 0 );
					}
				} else {
					dawufeg_theme_plugin_edit_constant_change();
				}
			} elseif ( 'first' === get_option( 'edit_first_time' ) && isset( $_GET['settings-updated'] ) ) {
				if ( defined( 'DISALLOW_FILE_EDIT' ) ) {
					if ( empty( get_option( 'dawufeg_editor' ) ) || 0 === get_option( 'dawufeg_editor' ) ) {

						// update constant to false.
						try {
							$config_transformer->update( 'constant', 'DISALLOW_FILE_EDIT', 'false', array( 'raw' => true ) );
							update_option( 'dawufeg_editor', 0 );
						} catch ( \Exception $ex ) {
							add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
							update_option( 'dawufeg_editor', 1 );
						}
					} else {

						// update constant to true.
						try {
							$config_transformer->update( 'constant', 'DISALLOW_FILE_EDIT', 'true', array( 'raw' => true ) );
							update_option( 'dawufeg_editor', 1 );
						} catch ( \Exception $ex ) {
							add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
							update_option( 'dawufeg_editor', 0 );
						}
					}
				} else {
					dawufeg_theme_plugin_edit_constant_change();
				}
			}
		}
	} else {

		// read constant value if defined and enable option based on that.
		if ( defined( 'DISALLOW_FILE_EDIT' ) ) {

			// if constant has valid value.
			if ( in_array( strtolower( strval( constant( 'DISALLOW_FILE_EDIT' ) ) ), array( '1', '' ), true ) ) {
				if ( 1 === intval( constant( 'DISALLOW_FILE_EDIT' ) ) ) {
					update_option( 'dawufeg_editor', 1 );
				} else {
					update_option( 'dawufeg_editor', 0 );
				}
			} else {
				update_option( 'dawufeg_editor', 0 );
			}
		} else {
			update_option( 'dawufeg_editor', 0 );
		}
	}

	// managing for file mods.
	if ( is_writable( $config_file_path ) && current_user_can( 'manage_options' ) ) {

		if ( defined( 'DISALLOW_FILE_MODS' ) ) {

			// set file mod to false by default.
			if ( 1 === intval( constant( 'DISALLOW_FILE_MODS' ) ) ) {
				try {
					$config_transformer->update( 'constant', 'DISALLOW_FILE_MODS', 'false', array( 'raw' => true ) );
				} catch ( \Exception $ex ) {
					add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
				}
			}
			if ( 'first' !== get_option( 'edit_first_time' ) ) {
				update_option( 'dawufeg_uploads', 1 ); // enable default options.
				dawufeg_theme_plugin_modification_constant_change();
				add_option( 'edit_first_time', 'first' );
			} else {
				dawufeg_theme_plugin_modification_constant_change();
			}
		} else {
			dawufeg_theme_plugin_modification_constant_change();
		}
	} else {

		// read constant value if defined and enable option based on that.
		if ( defined( 'DISALLOW_FILE_MODS' ) ) {
			if ( in_array( strtolower( strval( constant( 'DISALLOW_FILE_MODS' ) ) ), array( '1', '' ), true ) ) {
				if ( 1 === intval( constant( 'DISALLOW_FILE_MODS' ) ) ) {
					update_option( 'dawufeg_uploads', 1 );
				} else {
					update_option( 'dawufeg_uploads', 0 );
				}
			} else {
				update_option( 'dawufeg_uploads', 0 );
			}
		} else {
			update_option( 'dawufeg_uploads', 0 );
		}
	}

	if ( 'first' !== get_option( 'plugin_first_time' ) ) {
		update_option( 'dawufeg_plugin', 1 ); // default enable option for plugin auto update.
		update_option( 'dawufeg_theme', 1 ); // default enable option for theme auto update.
		add_option( 'plugin_first_time', 'first' );
	} elseif ( 'first' === get_option( 'plugin_first_time' ) ) {

		// checks the option auto update plugin and enable or disable it.
		if ( empty( get_option( 'dawufeg_plugin' ) ) || 0 === get_option( 'dawufeg_plugin' ) ) {
			add_filter( 'auto_update_plugin', '__return_true' );
		} else {
			add_filter( 'auto_update_plugin', '__return_false' );
		}

		// checks the option auto update theme and enable or disable it.
		if ( empty( get_option( 'dawufeg_theme' ) ) || 0 === get_option( 'dawufeg_theme' ) ) {
			add_filter( 'auto_update_theme', '__return_true' );
		} else {
			add_filter( 'auto_update_theme', '__return_false' );
		}
	}
}
