<?php
/**
 * Dynamically change capability for core update.
 *
 * @param string $allcaps .
 * @param string $caps .
 * @param mixed  $args .
 * @param string $user .
 * @return array
 */
function dawufeg_change_core_user_cap( $allcaps, $caps, $args, $user ) {
	$allcaps['update_core'] = false;
	return $allcaps;
}

/**
 * Allows automatic updates using filters.
 */
function dawufeg_update_constant_using_filter_hook() {
	$major_updates_op_cheks = true;
	$minor_updates_op_cheks = true;

	if ( 'first' !== get_option( 'install_first_time' ) ) {
		update_option( 'dawufeg_core_major', 1 );
		update_option( 'dawufeg_core_minor', 1 );
		add_option( 'install_first_time', 'first' );
	}

	if ( empty( get_option( 'dawufeg_core_major' ) ) || 0 === get_option( 'dawufeg_core_major' ) ) {
		$major_updates_op_cheks = false;
	}

	if ( empty( get_option( 'dawufeg_core_minor' ) ) || 0 === get_option( 'dawufeg_core_major' ) ) {
		$minor_updates_op_cheks = false;
	}

	// disable major core update.
	if ( $major_updates_op_cheks ) {
		add_filter( 'allow_major_auto_core_updates', '__return_false' );
		add_filter( 'user_has_cap', 'dawufeg_change_core_user_cap', 20, 4 );
	}

	// disable minor core update.
	if ( $minor_updates_op_cheks ) {
		add_filter( 'allow_minor_auto_core_updates', '__return_false' );
		add_filter( 'user_has_cap', 'dawufeg_change_core_user_cap', 20, 4 );
	}

	// disable core updates.
	if ( $major_updates_op_cheks && $minor_updates_op_cheks ) {
		add_filter( 'user_has_cap', 'dawufeg_change_core_user_cap', 20, 4 );
		add_filter( 'allow_dev_auto_core_updates', '__return_false' );
		add_filter( 'auto_update_core', '__return_false' );
		add_filter( 'wp_auto_update_core', '__return_false' );
	}
}

/**
 * WordPress update settings.
 */
function dawufeg_update_setting_callback() {
	// phpcs:disable WordPress.Security.NonceVerification.Recommended

	$config_file_path = strval( dawufeg_get_config_path() ); // get config file location.

	// if config file exist and writeable.
	if ( $config_file_path && file_exists( $config_file_path ) ) {
		if ( is_writable( $config_file_path ) && current_user_can( 'manage_options' ) ) {
			$config_transformer = new \WPConfigTransformer( $config_file_path );
		}
	}

	// if config file is writeable and defined in wp-config file.
	if ( defined( 'WP_AUTO_UPDATE_CORE' ) && ( is_writable( $config_file_path ) && current_user_can( 'manage_options' ) ) ) {

		// if plugin is installed first time.
		if ( 'first' !== get_option( 'first_time' ) ) {

			// on in-valid value of constant set constant value to false and remove core update.
			if ( ! in_array( strtolower( constant( 'WP_AUTO_UPDATE_CORE' ) ), array( '1', 'minor', 'major' ), true ) && defined( 'WP_AUTO_UPDATE_CORE' ) ) {

				try {
					$config_transformer->update( 'constant', 'WP_AUTO_UPDATE_CORE', 'false', array( 'raw' => true ) );
					update_option( 'dawufeg_core_major', 1 );
					update_option( 'dawufeg_core_minor', 1 );
					dawufeg_update_constant_using_filter_hook();
				} catch ( \Exception $ex ) {
					update_option( 'dawufeg_core_major', 0 );
					update_option( 'dawufeg_core_minor', 0 );
					add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
				}
			} elseif ( in_array( strtolower( constant( 'WP_AUTO_UPDATE_CORE' ) ), array( '1', 'minor', 'major' ), true ) && defined( 'WP_AUTO_UPDATE_CORE' ) ) {

				// if condition is true initially set to false by default.
				if ( 1 === intval( constant( 'WP_AUTO_UPDATE_CORE' ) ) ) {

					// chnage constnat to false.
					try {
						$config_transformer->update( 'constant', 'WP_AUTO_UPDATE_CORE', 'false', array( 'raw' => true ) );
						update_option( 'dawufeg_core_major', 1 );
						update_option( 'dawufeg_core_minor', 1 );
						dawufeg_update_constant_using_filter_hook();
					} catch ( \Exception $ex ) {
						update_option( 'dawufeg_core_major', 0 );
						update_option( 'dawufeg_core_minor', 0 );
						add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
					}
				} elseif ( 'major' === strtolower( constant( 'WP_AUTO_UPDATE_CORE' ) ) ) { // if major is constant value enable major update switch.
					update_option( 'dawufeg_core_major', 1 );
					dawufeg_update_constant_using_filter_hook();
				} elseif ( 'minor' === strtolower( constant( 'WP_AUTO_UPDATE_CORE' ) ) ) { // if minor is constant value enable minor update switch.
					update_option( 'dawufeg_core_minor', 1 );
					dawufeg_update_constant_using_filter_hook();
				}
			} else { // defaulting to false for core update.
				update_option( 'dawufeg_core_major', 1 );
				update_option( 'dawufeg_core_minor', 1 );
				dawufeg_update_constant_using_filter_hook();
			}
			add_option( 'first_time', 'first' );
		} elseif ( 'first' === get_option( 'first_time' ) && ! isset( $_GET['settings-updated'] ) ) {

			// on in-valid value of constant set constant to false.
			if ( ! in_array( strtolower( constant( 'WP_AUTO_UPDATE_CORE' ) ), array( '1', 'minor', 'major' ), true ) && defined( 'WP_AUTO_UPDATE_CORE' ) ) {

				// chnage constant to false.
				try {
					$config_transformer->update( 'constant', 'WP_AUTO_UPDATE_CORE', 'false', array( 'raw' => true ) );
					update_option( 'dawufeg_core_major', 1 );
					update_option( 'dawufeg_core_minor', 1 );
					dawufeg_update_constant_using_filter_hook();
				} catch ( \Exception $ex ) {
					update_option( 'dawufeg_core_major', 0 );
					update_option( 'dawufeg_core_minor', 0 );
					add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
				}
			} elseif ( in_array( strtolower( constant( 'WP_AUTO_UPDATE_CORE' ) ), array( '1', 'minor', 'major' ), true ) && defined( 'WP_AUTO_UPDATE_CORE' ) ) { // if value of constant is correct.

				if ( 1 === intval( constant( 'WP_AUTO_UPDATE_CORE' ) ) ) {
					update_option( 'dawufeg_core_major', 0 );
					update_option( 'dawufeg_core_minor', 0 );
				} elseif ( 'major' === strtolower( constant( 'WP_AUTO_UPDATE_CORE' ) ) ) { // if constant value is major.
					update_option( 'dawufeg_core_major', 1 );
					update_option( 'dawufeg_core_minor', 0 );
					dawufeg_update_constant_using_filter_hook();
				} elseif ( 'minor' === strtolower( constant( 'WP_AUTO_UPDATE_CORE' ) ) ) { // if constant value is minor.
					update_option( 'dawufeg_core_minor', 1 );
					update_option( 'dawufeg_core_major', 0 );
					dawufeg_update_constant_using_filter_hook();
				} else {
					update_option( 'dawufeg_core_major', 1 );
					update_option( 'dawufeg_core_minor', 1 );
					dawufeg_update_constant_using_filter_hook();
				}
			} else {
				dawufeg_update_constant_using_filter_hook();
			}
		} elseif ( 'first' === get_option( 'first_time' ) && isset( $_GET['settings-updated'] ) ) {

			// if constant is defined then update the constant value otherwise work with filters.
			if ( defined( 'WP_AUTO_UPDATE_CORE' ) ) {

				// if both major & minor option is selected update constant value to false.
				if ( get_option( 'dawufeg_core_major' ) && get_option( 'dawufeg_core_minor' ) ) {

					// update constant to false.
					try {
						$config_transformer->update( 'constant', 'WP_AUTO_UPDATE_CORE', 'false', array( 'raw' => true ) );
						dawufeg_update_constant_using_filter_hook();
					} catch ( \Exception $ex ) {
						add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
					}
				} elseif ( get_option( 'dawufeg_core_major' ) || get_option( 'dawufeg_core_minor' ) ) { // if major or minor selected update that value to constant.
					// set constant to major.
					if ( get_option( 'dawufeg_core_major' ) ) {
						try {
							$config_transformer->update( 'constant', 'WP_AUTO_UPDATE_CORE', 'major' );
							dawufeg_update_constant_using_filter_hook();
						} catch ( \Exception $ex ) {
							add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
						}
					}

					// set contant to minor.
					if ( get_option( 'dawufeg_core_minor' ) ) {
						try {
							$config_transformer->update( 'constant', 'WP_AUTO_UPDATE_CORE', 'minor' );
							dawufeg_update_constant_using_filter_hook();
						} catch ( \Exception $ex ) {
							add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
						}
					}
				} else {

					// update constant to true.
					try {
						$config_transformer->update( 'constant', 'WP_AUTO_UPDATE_CORE', 'true', array( 'raw' => true ) );
						dawufeg_update_constant_using_filter_hook();
					} catch ( \Exception $ex ) {
						add_action( 'admin_notices', 'dawufeg_error_update_notices_callback' );
					}
				}
			} else {
				dawufeg_update_constant_using_filter_hook();
			}
		}
	} else {
		dawufeg_update_constant_using_filter_hook();
	}
}
